
/********************************************************************************
 *										*
 *				 M A C E K					*
 *		("Matroids Also Computed Efficiently" Kit)			*
 *										*
 * A set of tools and routines for computations with representable matroids.	*
 * Copyright (C) 2001,2002  Petr Hlineny.					*
 *										*
 *   This program is free software; you can redistribute it and/or modify	*
 *   it under the terms of the GNU General Public License as published by	*
 *   the Free Software Foundation; either version 2 of the License, or		*
 *   (at your option) any later version.					*
 *   You should have received a copy of the GNU General Public License		*
 *   along with this program; if not, write to the Free Software		*
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.	*
 * 										*
 * See the top-level "ABOUT" and "doc/ *" files for more information.		*
 *										*
 ********************************************************************************/












/**
 * 
 * 
 * still under construction............
 * 	(a place for long exponential and brute-force matrix computations)
 * 
 * 
 * 
 * 
 * 
 * 
 * 
**/





#include "macek.h"  
#include "emx.h"







//*********** general-use function for listing all submatrices (as refering) of given properties......












/******************	Extended printing	*******************/
/******************************************************************/


//#undef CURDLEV
#define CURDLEV		7


#define maxx(a,b)	((a)<(b)?(b):(a))
#define maxxx(a,b,c)	maxx(maxx(a,b),c)
#define tripleix(ar,p,q,r)	ar[p][q][r]=ar[p][r][q]=ar[q][p][r]=ar[q][r][p]=ar[r][p][q]=ar[r][q][p]


/**
 * This is a function for an extended printing of a matroid.
 * Many interesting matroid-invariant numbers are printed here, derived from the
 * list of all bases, and from the closure operator, etc...
 * The output is meant to help you with identifying and comparing matroids,
 * but it has no other meaning in this program.
 * 
 * The amount of printed information is controlled by the value of lev.
 * 
**/

#define	EM_MAXTOPRINT	10

long	ematrix_printmore_ext(ematrix *e, int lev) {
	ematrix		**bas, **x, *ee;
	int		i,ii,iii, j,k, a,b, f1,f2,f3;
	int		basi[2*EM_MAXTOPRINT], basii[2*EM_MAXTOPRINT][2*EM_MAXTOPRINT],
			basiii[2*EM_MAXTOPRINT][2*EM_MAXTOPRINT][2*EM_MAXTOPRINT];
	long		hash = 0;
	
	if (!e || ROWSM(e)>=EM_MAXTOPRINT || COLSM(e)>=EM_MAXTOPRINT) {
		OUTPUT("Too big matroid for extended printing.\n");  return -1;
	}
	ee = ematrix_copy(e);
	for (a=0; a<2; a++) {
		for (i=0; i<ROWSM(ee); i++)
			if (ROWSID(ee,i)<=-EM_MAXTOPRINT || ROWSID(ee,i)>=EM_MAXTOPRINT) {
				OUTPUT("Line id labels out of range for extended printing %d.\n",ROWSID(ee,i));  return -1; }
		ematrix_transpose(ee);
	}
	for (i=0; i<2*EM_MAXTOPRINT; i++)  basi[i] = 0;
	for (i=0; i<2*EM_MAXTOPRINT; i++) for (ii=0; ii<2*EM_MAXTOPRINT; ii++)  basii[i][ii] = 0;
	if (lev>0)  SOUTPUT("\n");
	hash = ROWSM(ee)+10*COLSM(ee);
	hash = 90909l*hash+111l*hash*hash*hash;
	
		/**
		 * Here we collect and print all bases - their total number, and numbers
		 * per elements and element pairs.
		 * We also add the numbers to the matroid hash value.
		 * The bases are also used later, and they are freed at the end.
		**/
	bas = struct_allbases(ee);
	hash += alist_getlength(bas)*10101l;
	if (lev>=0)  OUTPUT("Number of %p [%s] bases  %d\n",e,EMNAME(e)?EMNAME(e):"",alist_getlength(bas));
	for (x=bas; *x; x++) 
	  for (i=0; i<ROWSM(ee); i++) {
		j = ROWSID(*x,i)+EM_MAXTOPRINT;  j = ((j>=0&&j<2*EM_MAXTOPRINT)? j:EM_MAXTOPRINT);
		basi[j]++;
		for (ii=0; ii<i; ii++) {
			k = ROWSID(*x,ii)+EM_MAXTOPRINT;  k = ((k>=0&&k<2*EM_MAXTOPRINT)? k:EM_MAXTOPRINT);
			basii[j][k]++;  basii[k][j]++;
		}
	}
	if (basi[EM_MAXTOPRINT]!=0)  {PROGERROR("Something wrong - incorrect line id labels??");}
	for (i=0; i<2*EM_MAXTOPRINT; i++) {
		hash += 505l*basi[i]+7l*basi[i]*basi[i];
		for (ii=0; ii<i; ii++)  hash += 13l*basii[i][ii]+1l*basii[i][ii]*basii[i][ii];
		if (basii[i][ii]!=basii[ii][i])  {PROGERROR("Something wrong - nonsymmetric basis pairs!");}
	}
	if (lev>0) {
		OUTPUT(" - by elements ");
		for (i=0; i<2*EM_MAXTOPRINT; i++)  if (basi[i]>0)
			SOUTPUT(" [%d: %d]",i-EM_MAXTOPRINT,basi[i]);
		SOUTPUT("\n");
	}
	if (lev>1) {
		OUTPUT(" - by element pairs\n");  a = 1;
		for (i=0; i<2*EM_MAXTOPRINT; i++) {
			if (a)  SOUTPUT("    ~\t ");  a = 0;
			for (ii=0; ii<2*EM_MAXTOPRINT; ii++)  if (basii[i][ii]>0) {
				a = 1;
				SOUTPUT("[%2d'%2d: %-3d]",i-EM_MAXTOPRINT,ii-EM_MAXTOPRINT,basii[i][ii]);
			}
			if (a)  SOUTPUT("\n");
		}
		SOUTPUT("\n");
	}
	
		/**
		 * Here we list all lines, and other flats later........
		 * 
		**/
	for (i=0; i<2*EM_MAXTOPRINT; i++) for (ii=0; ii<2*EM_MAXTOPRINT; ii++)  basii[i][ii] = 0;
	if (lev>1)  OUTPUT("Listing flats - lines:\n");
	f2 = 0;
	if (lev>1)  for (x=bas; *x; x++) {
		for (i=0; i<ROWSM(ee); i++)  for (ii=0; ii<i; ii++) {
			b = 1;
				/* must prevent repetition of the same lines (smallest rows, and marks) */
			k = maxx(ROWSID(*x,i),ROWSID(*x,ii));
			if (basii[ROWSID(*x,i)+EM_MAXTOPRINT][ROWSID(*x,ii)+EM_MAXTOPRINT])
				continue;
			basii[ROWSID(*x,i)+EM_MAXTOPRINT][ROWSID(*x,ii)+EM_MAXTOPRINT] = basii[ROWSID(*x,ii)+EM_MAXTOPRINT][ROWSID(*x,i)+EM_MAXTOPRINT] = 1;
				/* the first cycle only checks whether there is a dependent column, plus all bigger labels */
			for (j=0; j<COLSM(*x) && b>0; j++)
				if (ematrix_linezero_col3(*x,j,i,ii,-1)>=0) {
					if (COLSID(*x,j)<=k)  b = 0;  else  b++;
				}
			if (b>1)  EMATDEBUG(CURDLEV+2,*x,"\t\t.\t");
				/* the second cycle possibly prints the whole line */
			if (b>1)  SOUTPUT("    ~\tline (%d)\t%2d, %2d, ",f2+1,ROWSID(*x,i),ROWSID(*x,ii));
			if (b>1)  for (j=0; j<COLSM(ee); j++)
				if (ematrix_linezero_col3(*x,j,i,ii,-1)>=0)
					SOUTPUT("%2d, ",COLSID(*x,j));
			if (b>1)  SOUTPUT("\n");
			if (b>1)  f2++;		/* counts the number of lines */
		}
	}
	if (lev>1)  OUTPUT("Line listing is not yet correct for not 3-connected!\n");
#if 0
	
	//************* wrong - missing flats if the smallest pair/triple is dependent!!!!!!!!!!
	// - remember dependent flats by the line-labels, then reject the bigger
	//   non-cyclic flats (brute force, but fast)............. - separate function
	
	hash += 77*f2+f2*f2*f2;
		/**
		 * 
		 * 
		**/
	for (i=0; i<2*EM_MAXTOPRINT; i++) for (ii=0; ii<2*EM_MAXTOPRINT; ii++)
		for (iii=0; iii<2*EM_MAXTOPRINT; iii++)  basiii[i][ii][iii] = 0;
	if (lev>2)  OUTPUT("Listing flats - planes:\n");
	f3 = 0;
	if (lev>2)  for (x=bas; *x; x++) {
		for (i=0; i<ROWSM(ee); i++)  for (ii=0; ii<i; ii++)  for (iii=0; iii<ii; iii++) {
			b = 1;
				/* must prevent repetition of the same planes (smallest rows, and marks) */
			k = maxxx(ROWSID(*x,i),ROWSID(*x,ii),ROWSID(*x,iii));
			if (basiii[ROWSID(*x,i)+EM_MAXTOPRINT][ROWSID(*x,ii)+EM_MAXTOPRINT][ROWSID(*x,iii)+EM_MAXTOPRINT])
				continue;
			tripleix(basiii,ROWSID(*x,i)+EM_MAXTOPRINT,ROWSID(*x,ii)+EM_MAXTOPRINT,ROWSID(*x,iii)+EM_MAXTOPRINT) = 1;
				/* the first cycle only checks whether there is a dependent column, plus all bigger labels */
			for (j=0; j<COLSM(*x) && b>0; j++)
				if (ematrix_linezero_col3(*x,j,i,ii,iii)>=0) {
					if (COLSID(*x,j)<=k)  b = 0;  else  b++;
				}
			if (b>1)  EMATDEBUG(CURDLEV+2,*x,"\t\t.\t");
				/* the second cycle possibly prints the whole line */
			if (b>1)  SOUTPUT("    ~\tplane (%d)\t%2d, %2d, %2d, ",f3+1,ROWSID(*x,i),ROWSID(*x,ii),ROWSID(*x,iii));
			if (b>1)  for (j=0; j<COLSM(ee); j++)
				if (ematrix_linezero_col3(*x,j,i,ii,iii)>=0)
					SOUTPUT("%2d, ",COLSID(*x,j));
			if (b>1)  SOUTPUT("\n");
			if (b>1)  f3++;		/* counts the number of lines */
		}
	}
	hash += 55*f3+f3*f3*f3;
#endif
	
		/**
		 * So far, the matroid hash-value collects information about rank,
		 * number of bases, numbers of bases per each element and each pair of elements.
		 * 
		 * ****** NO ****** and the total numbers of flats of ranks up to 3.
		 * 
		 * You must update the version number if you change the collected information!
		**/
	if (lev>0)  SOUTPUT("\n");
	if (lev>=0)  OUTPUT("Overall matroid %p [%s] hash-value (version 1.0):  %ld\n",e,EMNAME(e)?EMNAME(e):"",hash);
	
	if (bas)  dispose_alist_mats(bas);
	dispose_ematrix(ee);
	return hash;
}



//*********** printing all separations up to the given size here............


























