
/********************************************************************************
 *										*
 *				 M A C E K					*
 *		("Matroids Also Computed Efficiently" Kit)			*
 *										*
 * A set of tools and routines for computations with representable matroids.	*
 * Copyright (C) 2001,2002  Petr Hlineny.					*
 *										*
 *   This program is free software; you can redistribute it and/or modify	*
 *   it under the terms of the GNU General Public License as published by	*
 *   the Free Software Foundation; either version 2 of the License, or		*
 *   (at your option) any later version.					*
 *   You should have received a copy of the GNU General Public License		*
 *   along with this program; if not, write to the Free Software		*
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.	*
 * 										*
 * See the top-level "ABOUT" and "doc/ *" files for more information.		*
 *										*
 ********************************************************************************/










/**
 * This header contains the debugging defines and checks...
 * 
**/


#ifndef	DEBUG_H
#define	DEBUG_H








/******************	"Debug" definitions	*******************/



extern	int	junk;		/* general use for non-important assignments... */
extern	char    *ssss,  debugoutbuf[10000];

extern	int	uerror_occured, perror_occured;
extern	int	crash_all_errors;

extern	FILE	*errorout, *debugout, *printout;




//#define	CRASH			(fsync(fileno(debugout)),fsync(fileno(errorout)),mmmm=NULL,*(int*)mmmm=0)
#define	CRASH			0

#define	DUPLERROR(out)		(errorout!=(out) && (!isatty(fileno(out)) || !isatty(fileno(errorout))))
#define	DERRORPRINT(args...)	(fprintf(errorout,args),DUPLERROR(debugout)?fprintf(debugout,args):0,\
				 (printout!=debugout && DUPLERROR(printout))?fprintf(printout,args):0)

#define PROGERROR(args...)	(sprintf(debugoutbuf,"\n****** Program ERROR [bad:-(]: %.25s, in %.25s() line %d ******\n  ",__FILE__,__FUNCTION__,__LINE__),\
				 snprintf(debugoutbuf+strlen(debugoutbuf),5000,args),\
				 sprintf(debugoutbuf+strlen(debugoutbuf),"\n================================================================\n"),\
				 DERRORPRINT(debugoutbuf),(void)(crash_all_errors?CRASH:0),perror_occured=-1)
#define PROGERROREXIT(args...)	(crash_all_errors=1,PROGERROR(args),exit(-1),-1)

#define USERERROR(args...)	(sprintf(debugoutbuf,"*** ERROR: (reported by %.25s, in %.25s() l.%d) ***\n  ",__FILE__,__FUNCTION__,__LINE__),\
				 snprintf(debugoutbuf+strlen(debugoutbuf),5000,args),sprintf(debugoutbuf+strlen(debugoutbuf),"\n"),\
				 DERRORPRINT(debugoutbuf),uerror_occured=-1)
#define USERERROREXIT(args...)	(USERERROR(args),exit(-1),-1)

#define USERERRORWH(wh,wl,args...)	(sprintf(debugoutbuf,"*** ERROR (by %.12s, in %.15s() %d) in \"%.20s\" l.%d: ***\n  ",\
					 __FILE__,__FUNCTION__,__LINE__,(wh),(wl)),\
					 snprintf(debugoutbuf+strlen(debugoutbuf),5000,args),sprintf(debugoutbuf+strlen(debugoutbuf),"\n"),\
					 DERRORPRINT(debugoutbuf),uerror_occured=-1)




#define	OUTPUT(args...)		(fprintf(printout,"~%3.3d~\t",TIME000),SOUTPUT(args))
#define	SOUTPUT(args...)	(fprintf(printout,args))

#define EMATOUTPUT(e,pf)	ematrix_fprint_pref(printout,e,pf)
#define EMATOUTPUTS(e,pf)	ematrix_fprint_nofr(printout,e,pf)

#define FRMOUTPUT(f,pf)		frame_fprint_all(printout,f,pf)
#define FROUTPUT(f,pf)		frame_fprint(printout,f,pf)
#define FROUTPUTS(f,pf)		frame_fprint_inf(printout,f,pf)



extern int	printlev;
#define	TESTDLEV(l)		(((l)<=DEBUGLEV && (l)<=printlev) || (l)<=0)

#define	DEBUG(l,args...)	(junk = TESTDLEV(l)? \
					(fprintf(debugout,"%*s[%-8.8s:%17.17s()%-4d~%3.3d] ",\
					((l)>2?(l)-2:(l)),((l)>2?"":"#"),\
					__FILE__,(ssss=__FUNCTION__,ssss+0),__LINE__,TIME000),\
					fprintf(debugout,args),0):0)
#define	SDEBUG(l,args...)	(junk = TESTDLEV(l)? (fprintf(debugout,args),0):0)

#define	EMATDEBUG(l,e,pf)	(junk = TESTDLEV(l)? (ematrix_fprint_pref(debugout,e,pf),0):0)
#define	EMATDEBUGS(l,e,pf)	(junk = TESTDLEV(l)? (ematrix_fprint_nofr(debugout,e,pf),0):0)

#define	FRDEBUG(l,f,pf)		(junk = TESTDLEV(l)? (frame_fprint(debugout,f,pf),0):0)
#define	FRDEBUGS(l,f,pf)	(junk = TESTDLEV(l)? (frame_fprint_brief(debugout,f,pf),0):0)

#define	ELIMQDEBUG(l,q,pf)	(junk = TESTDLEV(l)? (elimseqm_fprint_full(debugout,q,pf),0):0)
#define	ELIMQDEBUGS(l,q,pf)	(junk = TESTDLEV(l)? (elimseqm_fprint_pref(debugout,q,pf),0):0)




#define	DEBUGLEV	6

#define	IFRANDDEBUG(f)		(DEBUGLEV>=CURDLEV || (DEBUGLEV>=CURDLEV-3 && RANDOM()%(f)==1))
#define	IFRANDDEBUGLESS(f)	(DEBUGLEV>=CURDLEV+2 || (DEBUGLEV>=CURDLEV-3 && RANDOM()%(f)==1))


#define	DECRCHE(c,d)	((c)<-5? -1:((c)<0?(c): ((c)-(d)<0?0: (c)-(d))))
	/* (ch==-7 is used for randomized tests, so should not be accidentally entered!) */
#define	DECRCH(c)	DECRCHE(c,1)


int begin_check(void) ;
int final_check(void) ;







/******************	"Performance" settings	*******************/


	/* uncomment the following to skip (almost) all debug tests and printouts in the program */
/*#define	FASTPROG	*/


/* the following definitions duplicate (in one place) default definitions from other headers */


	/* the number of allocated matrices to keep for recycling (ematrix.c) */
/*#define	EM_ALCMAX	4000*/

	/* the size of the table of arbitrary-list ends (misc.c) */
/*#define	ALISTHASHSZ	2507*/



	/* whether to use cached automorphisms and matrix bases (struct.h, straut.c) */
/*#define	CACHEDAUTS	1*/
/*#define	STR_MAXAUTCACHE	2013*/

/*#define	CACHEDBASES	1*/

	/* the maximal number of bases generated for one matrix (strmin.c) */
/*#define	STR_MAXBASES	200000*/

	/* the maximal number of displayed minors generated for one matrix (strmin.c) */
/*#define	STR_MAXMINORS	30000*/

	/* the maximal number of automorphisms generated for one matrix (straut.c) */
/*#define	STR_MAXAUTOMS	20000*/



	/* a speedup choice - use randomized structure checks at level 1 for at least this
	    number of elements (gen.h, genstep.c) */
/*#define GEN_RANDSTRUCTCH	10*/

	/* a speedup choice - use subsequence pre-comparing in canonical checks (gen.h, genstep.c) */
/*#define GEN_SUBSEQCOMP	1*/

	/* a speedup choice - use randomized canonical checks at level 1 for at least this
	    number of elements (gen.h, genstep.c) */
/*#define GEN_RANDCANON		10*/


	/* whether to use pfield pre-checking when generating extensions (gen.h, ematext.c) */
/*#define GEN_USEEXTPREPF	1*/









/******************	...	*******************/



#ifdef	FASTPROG
#define DEBUGLEV	-1
#define CRASH		0

#define DEBUG(l,args...)	junk=0
#define SDEBUG(l,args...)	junk=0

#define EMATDEBUG(l,e,pf)	junk=0
#define EMATDEBUGS(l,e,pf)	junk=0
#define FRDEBUG(l,f,pf)		junk=0
#define FRDEBUGS(l,f,pf)	junk=0
#define ELIMQDEBUG(l,q,pf)	junk=0
#define ELIMQDEBUGS(l,q,pf)	junk=0

#define	IFRANDDEBUG(f)		0
#define	IFRANDDEBUGLESS(f)	0
#endif


















#endif	/* (of #ifndef DEBUG_H) */















