
/********************************************************************************
 *										*
 *				 M A C E K					*
 *		("Matroids Also Computed Efficiently" Kit)			*
 *										*
 * A set of tools and routines for computations with representable matroids.	*
 * Copyright (C) 2001,2002  Petr Hlineny.					*
 *										*
 *   This program is free software; you can redistribute it and/or modify	*
 *   it under the terms of the GNU General Public License as published by	*
 *   the Free Software Foundation; either version 2 of the License, or		*
 *   (at your option) any later version.					*
 *   You should have received a copy of the GNU General Public License		*
 *   along with this program; if not, write to the Free Software		*
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.	*
 * 										*
 * See the top-level "ABOUT" and "doc/ *" files for more information.		*
 *										*
 ********************************************************************************/













/**
 * 
 * miscelanelous, general-use functions..........
 * 
 * 
**/





int     macek_begin(void) ;
int     macek_final(void) ;









/******************	Handling arbitrary lists	*******************/
/**************************************************************************/


/**
 * A "list" is a structure of general type  **list_type, terminated by NULL.
 * (However, to avoid pointer compatability problems, we transfer lists also as *void...)
 * To allow lists to grow, we store additional information right after the terminating NULL:
 *  If ls[i]==NULL, then ls[i+1] is a pointer to the start of the list, and ls[i+2] (as long)
 *  is the length remaining after the end of the list (minus the terminating NULL and these two).
 * A special magic number is stored into one element just before the list starts.
 * Moreover, a list may be given pointing to an arbitrary internal element, and the code
 * here can recover the list's true start and end.
 * 
 * No internal element of a list may be set to NULL !!!
**/


/**
 * Here we create and dispose lists - these functions are for general use.
 * - new_alist() creates a new empty list of the given initial size.
 * - alist_free() frees the given list (possible to give any element as a pointer), but NOT the
 *  list elements.
 * - dispose_alist_ext() frees all the given list elements (using an optional disposing function
 *  given in *ff), and then frees the list itself.
 *  (All list elements from the start of the list are freed !!!)
**/

void*   new_alist(long sz) ;

void    alist_free(void *ls) ;	/* (this does not free the list elements!) */

void    dispose_alist_ext(void *ls, void (*ff)(void*)) ;
void    dispose_alist(void *ls) ;	/* (no dispose function for the elements given, so free() is used) */

#define	dispose_alist_copts(ls)	dispose_alist_ext(ls,(void (*)(void*))&dispose_comoption)
#define	dispose_alist_mats(ls)	dispose_alist_ext(ls,(void (*)(void*))&dispose_ematrix)
#define	dispose_alist_frams(ls)	dispose_alist_ext(ls,(void (*)(void*))&dispose_frame_ls)


/**
 * Here we get the start position of a list (recall than a list may be referred to
 * by any internal position), and the total length.
 * 
 * Then a function appends a new element ap to the end of the given list.
 * (Again, it is possible to give any element of the list as a pointer ls.)
 * If the list is not sufficiently large, it is reallocated to twice its size.
 * The list is returned back (possibly different address, so it must be stored!).
 * 
 * Next similar function appends a whole list apl to the given list ls, and frees
 * the list apl (not its elements).
**/

void*   alist_getstart(void *ls) ;
int     alist_getlength(void *ls) ;

void*   alist_append(void *ls, void *ap) ;
void*   alist_applist(void *ls, void *apl) ;


/**
 * This function deletes one element pointed by ls from the list (the element itself is not freed).
 * The remaining elements in the list are simply shifted by one to the front.
 * The list is returned back (address is unchanged).
**/

void*   alist_delete(void *ls) ;
#define	alist_delete_last(ls)	alist_delete(((void**)(ls))+alist_getlength(ls)-1)

/**
 * This function inserts one element to the position pointed by ins in the list.
 * The list is possibly enlarged like in alist_append(), and the remaining elements
 * are shifted by one to the end.
 * The list is returned back (possibly different address, so it must be stored!).

 * This function returns a copy of the given list.
 * (The list ls may be given by an arbitrary internal position.)
**/

void*   alist_insert(void *ls, void *ins, void *ai) ;

void*   alist_copy(void *ls) ;







/******************	Other...................	*******************/
/**************************************************************************/



/**
 * 
 * 
 * 
 * 
**/


char*   fname_extract(char *fn) ;

char*   fdir_extract(char *fn) ;


FILE*   fopen_path(char *fn, char *mod, char **path, char *dir, char *ext) ;

FILE*   fopen_dir(char *fn, char *mod, int pre) ;
































