
/********************************************************************************
 *										*
 *				 M A C E K					*
 *		("Matroids Also Computed Efficiently" Kit)			*
 *										*
 * A set of tools and routines for computations with representable matroids.	*
 * Copyright (C) 2001,2002  Petr Hlineny.					*
 *										*
 *   This program is free software; you can redistribute it and/or modify	*
 *   it under the terms of the GNU General Public License as published by	*
 *   the Free Software Foundation; either version 2 of the License, or		*
 *   (at your option) any later version.					*
 *   You should have received a copy of the GNU General Public License		*
 *   along with this program; if not, write to the Free Software		*
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.	*
 * 										*
 * See the top-level "ABOUT" and "doc/ *" files for more information.		*
 *										*
 ********************************************************************************/














/**
 * Some theory for start... about equivalence of matrices, tight majors, etc :
 * (Read general description of "structural" functions on the top of ../include/struct.h.)
 * 
 * Here we write several more functions extending the minor-related functions
 * from strmin.c
 * 
 * ...............
**/






#include "macek.h"
#include "str.h"











/******************	Finding matrix equivalence	*******************/
/**************************************************************************/


//#undef CURDLEV
#define CURDLEV		7

static int	inrecur = 0;	/* (used for debug-testing below...) */


/**
 * This function tests whether the matrices e1,e2 are equal up to scale of rows and columns.
 * (Line-permutations are not allowed here.)
 * The function estruct_fitcols() is called to test a column fitting with scale.
 * If ch>0, then some more information are printed.
 * Returns 1 if a scaling is found, and 0 if not.
 * Additionally, if ao!=NULL, then the resulting scales (of e1 to match e2) are stored to
 * the automorphism structure ao, which must be given allocated(!).
**/

#define	MXSZ	(NUM_LONG_BITS-2)	/* (must fit into long bits - def from straut.c) */

int	struct_scaledequal_ext(int ch, ematrix *e1, ematrix *e2, emxaut *ao) {
	int	i,r, pc[MXSZ];
	long	hc[MXSZ];
	exp_t	xrl[MXSZ],xcl[MXSZ], *xr,*xc;
	sign_t	grl[MXSZ],gcl[MXSZ], *gr,*gc;
	void	*data;

	if (ch<-1)  {PROGERROR("Do not call with ch==%d <-1",ch);}
	DEBUG(CURDLEV-1,"Testing matrices %p and %p for scaled equality...\n",e1,e2);
	if (ch>0) {
		EMATDEBUG(CURDLEV+1,e1,"\t~~\t");  EMATDEBUG(CURDLEV+1,e2,"\t~~\t");
	}
	if (ROWSM(e1)!=ROWSM(e2) || COLSM(e1)!=COLSM(e2)) {
		DEBUG(CURDLEV-2,"...  the matrices have different dimensions - NO.\n");
		return 0;
	}
	if (ao) {		/* if the line scales should be stored in given ao structure */
		xr = (ao)->xr;  xc = (ao)->xc;  gr = (ao)->gr;  gc = (ao)->gc;
	} else {
		xr = xrl;  xc = xcl;  gr = grl;  gc = gcl;
	}
	for (i=0; i<MXSZ; i++)  hc[i] = i+1;	/* to force unique positions of all columns */
	
				/* testing column fit - no permuting, just scaling of lines */
	r = struct_fitcols(1,e1,e2,1,hc,hc,NULL,NULL, pc,xr,gr,xc,gc,&data);
	if (r>=0)  r = struct_fitcols(0,e1,e2,1,hc,hc,NULL,NULL, pc,xr,gr,xc,gc,&data);
	struct_fitcols(-1,e1,e2,1,hc,hc,NULL,NULL, pc,xr,gr,xc,gc,&data);
	
#ifndef FASTPROG
	if (!inrecur)  DEBUG(CURDLEV-2*(r<0),"...  testing matrices for scaled equality found %s.\n",r>=0?"YES":"NO");
	if (IFRANDDEBUG(36) && !inrecur) {
		inrecur = 1;		/* (debug-testing the symmetric dual situation) */
		ematrix_transpose(e1);  ematrix_transpose(e2);
		if (struct_scaledequal_ext(0,e2,e1,NULL)!=(r>=0))  {PROGERROR("The result must be symmetric! r=%d",r);}
		ematrix_transpose(e1);  ematrix_transpose(e2);
		inrecur = 0; }
#endif
	return (r>=0? 1:0);
}



/**
 * This function tests whether the matrices e1,e2 are equivalent by pivoting, permuting and
 * scaling; i.e. equivalence is tested over all possible bases of e1 against e2.
 * (Nonequivalent representations of the same matroid are NOT "equivalent" in this function.)
 * The function estruct_dispminor() is called to test a mapping for each basis.
 * 
 * If ch>0, then some more information are printed, and >1 extra paranoic (debug) tests performed.
 * If ch<0 (ch<2 if bs,rbs==NULL), then we only ask for an existence of an equivalence.
 * It returns 1 if an equivalence is found, and 0 if not.
 * Additionally, if bs,rbs!=NULL, then (some) basis of e1 displaying e2 is stored into *bs,
 * and (some) refering matrix showing lines of e2 in *bs is stored in *rbs.
**/

int	struct_equivalent_ext(int ch, ematrix *e1, ematrix *e2, ematrix **bs, ematrix **rbs) {
	ematrix		**bas,**b, **bmin;
	int		has=0;
	
	if (ch<-1)  {PROGERROR("Do not call with ch==%d <-1",ch);}
	DEBUG(CURDLEV-1,"Testing matrices %p and %p for equivalence...\n",e1,e2);
	if (ch>0) {
		EMATDEBUG(CURDLEV+1,e1,"\t~\t");  EMATDEBUG(CURDLEV+1,e2,"\t~\t");
	}
	if (ROWSM(e1)!=ROWSM(e2) || COLSM(e1)!=COLSM(e2)) {
		DEBUG(CURDLEV-2,"...  the matrices have different dimensions - NO.\n");
		return 0;
	}
	if (ch<0 || (ch<2 && !bs && !rbs)) {		/* simply calling the hasminor function */
		has = (struct_hasminor(e1,e2)>0);
	} else {					/* if requested generating the equivalences */
		bas = struct_allbases_ch(DECRCH(ch),e1);
		has = 0;
		for (b=bas; *b && !has; b++) {
			DEBUG(CURDLEV+1,"# matrix base %p testing for equivalence\n",*b);
			if (ch>1)  EMATDEBUG(CURDLEV+2,*b,"\t\t~\t");
			bmin = struct_dispminor_ch(DECRCHE(ch,2),*b,e2);
			has = (bmin?bmin[0]!=NULL:0);
			if (has) {			/* (got nonempty list of minors) */
				if (ch>0)  EMATDEBUG(CURDLEV+1,bmin[0],"\t\t\t>>\t");
				if (bs)  *bs = ematrix_copy(*b);
				if (rbs)  *rbs = ematrix_copy(bmin[0]);
			}				/* should record the equivalence in bs,rbs */
			if (bmin)  dispose_alist_mats(bmin);
		}
		dispose_alist_mats(bas);
		struct_printdispm = NULL;  struct_printdispp = NULL;
	}
#ifndef FASTPROG
	if (!inrecur)  DEBUG(CURDLEV-2*(!has),"...  testing matrix equivalence %p,%p found %s.\n",e1,e2,has?"YES":"NO");
	if (IFRANDDEBUG(36) && !inrecur) {
		inrecur = 1;		/* (debug-testing the symmetric situation) */
		ematrix_transpose(e1);  ematrix_transpose(e2);
		if (struct_equivalent_ext(0,e2,e1,NULL,NULL)!=(!!has))  {PROGERROR("The result must be symmetric! has=%d",has);}
		ematrix_transpose(e1);  ematrix_transpose(e2);
		inrecur = 0; }
#endif
	return has? 1:0;
}


//**************** make estruct_stronglyequiv() as well ?? (allowed permutations)
















/******************	Checking tight majors	*******************/
/******************************************************************/


#undef CURDLEV
#define CURDLEV		7



/**
 * This function tests whether the matrix e has an element that can be both contracted and
 * deleted keeping some minors from the list tghl[] (NULL-terminated) - "removable" element.
 * (If not, then the matroid of e is a "tight major" of the list tghl[].)
 * As a "minor" we mean a displayed subrepresentation in an equivalent matrix to em
 *  - see struct_dispminor_..().
 * All bases (unless ch==-7) of e are tested against all minors in tghl[] in this function.
 * 
 * If ch>0+, then some more information are printed, and paranoic (debug) tests performed if ch>=0.
 * If ch==-7, then only a randomized test is performed - for only a sublist of bases.
 * The function returns 1 if a removable (del+contr) element is found, and 0 if not.
 * 
 * It is possible to request printing the removable element by struct_hasdelcontr_printit().
**/

#define MXTG    (NUM_LONG_BITS/2-2)	/* (the interval -MXTG...MXTG must fit into long bits) */


int	struct_hasdelcontr_ext(int ch, ematrix *em, ematrix **tghl, int *which) {
	ematrix		*ee, **b,**x, **bas, **bmin;
	int		i,ii,j,jj, ret;
	long		fpatr,fpatc, fr,fc, fx,fxr,fy;
	char		*rands=NULL;
	extern int	callhash, rejhash, rejrowf, rejcolf;	/* (for debugging statistics) */
	
	for (i=0; i<ROWSM(em)+COLSM(em); i++) {
		j = i<ROWSM(em)? ROWSID(em,i): COLSID(em,i-ROWSM(em));
		if (j<-MXTG || j>MXTG || j==0)
			{PROGERROR("cannot work with line id %d out of range %d...%d or 0!",j,-MXTG,MXTG);}
	}
	if (which)  *which = 0;
	rands = ch<=-7? "randomized":"";
#ifndef	FASTPROG
	DEBUG(CURDLEV-1,"Testing mat %p for deletion+contraction (removable) elem %s...\n",em,rands);
	if (IFRANDDEBUG(16))
	  for (i=0; i<ROWSM(em)+COLSM(em); i++)  for (ii=i+1; ii<ROWSM(em)+COLSM(em); ii++) {
			j = i<ROWSM(em)? ROWSID(em,i): COLSID(em,i-ROWSM(em));
			jj = ii<ROWSM(em)? ROWSID(em,ii): COLSID(em,ii-ROWSM(em));
			if (i!=ii && j==jj)  {PROGERROR("cannot handle two lines with the same id (%d, %d)",i,ii);}
		}
	if (ch>0 || perror_occured)  EMATDEBUG(CURDLEV+1,em,"\t=\t");
	for (i=0; ch>1 && tghl[i]; i++) {
		EMATDEBUG(CURDLEV+3,tghl[i],"\t\t`\t");
	}
	DEBUG(CURDLEV+1,"   - testing against %d minors for a tight major\n",i);
#endif	
	if (tghl?!tghl[0]:1)  return 0;	/* nothing to test here for empty "tight" list */
	
	for (i=ii=j=0; tghl[j]; j++) {	/* a simple size-based test first */
		if (ROWSM(em)>ROWSM(tghl[j]))  i = 1;
		if (COLSM(em)>COLSM(tghl[j]))  ii = 1;
	}
	if (i==0 || ii==0) {
		DEBUG(CURDLEV-1," -> not enough large em to have a del+contr element (j=%d, i=%d, ii=%d)\n",j,i,ii);
		return 0;
	}
	callhash = rejhash = rejrowf = rejcolf = 0;
	
			/**
			 * We look at all tghl[ii]-minors in all basis of the given matrix em.
			 * For each minor, we record, in fpatr and fpatc, what rows 
			 * and what columns of the basis-matrix can be removed while
			 * keeping the respective minor.
			 * The matrix lines are identified by their id tags (+-number).
			 * Finally, we check the recorded patterns in fpatr,fpatc against each
			 * other to find a common intersection (=> not tight).
			**/
	fpatr = fpatc = 0l;
	ret = 0;
	if (ch>-7)  bas = struct_allbases_ch(DECRCH(ch),em);
	else  bas = struct_allbases_rand(em);	/* (for ch==-7, we test only a randomized list of bases - faster) */
	for (b=bas; *b && !ret; b++)
	  for(ii=0; tghl[ii] && !ret; ii++) {
					/* here we generate all displayed minors */
		bmin = struct_dispminor_ch(ch<-1?-1:DECRCH(ch),*b,tghl[ii]);
		if (!bmin)  continue;
		if (bmin[0]!=NULL || DEBUGLEV>=CURDLEV+4)
			DEBUG(CURDLEV+3,"# matrix base %p against a tight %d minor\n",*b,ii);
		if (ch>1 && ii==0 && bmin[0]!=NULL)  EMATDEBUGS(CURDLEV+3,*b,"\t\t\t\t");
		
		fr = fc = 0l;
		for (i=0; i<ROWSM(*b); i++)  fr |= (1l<<(ROWSID(*b,i)+MXTG));
		for (j=0; j<COLSM(*b); j++)  fc |= (1l<<(COLSID(*b,j)+MXTG));
		for (x=bmin; *x; x++) {
			fx = fr;	/* and here we record the removable lines for the minor */
			for (i=0; i<ROWSM(*x); i++) {
				fy = (1l<<(ROWSID(*x,i)+MXTG));
				if ((fy&fx)==0)  {PROGERROR("wrong minor row id %d",ROWSID(*x,i));}
				fx -= fy;
			}
			fpatr |= (fxr = fx);
			fx = fc;
			for (j=0; j<COLSM(*x); j++) {
				fy = (1l<<(COLSID(*x,j)+MXTG));
				if ((fy&fx)==0)  {PROGERROR("wrong minor col id %d",COLSID(*x,j));}
				fx -= fy;
			}
			fpatc |= fx;
			DEBUG(CURDLEV+3,"  : row and column patterns %lX, %lX\n",fxr,fx);
			if (ch>1)  EMATDEBUGS(CURDLEV+3,*x,"\t\t\t\t>>\t");
		}
		dispose_alist_mats(bmin);
		if ((fpatr&fpatc)!=0)  ret = 1;
	}
	if (ch>-7) {	/* when dealing with full list of bases, we may "recycle" it for further use */
		struct_allbases_recycle(bas);
		bas = NULL;
	} else {
		dispose_alist_mats(bas);
	}
			/* finally, find which is the removable element id */
	if (ret)  for (i=-MXTG; i<=MXTG; i++)  if ((1l<<(i+MXTG))&fpatr&fpatc)  break;
	if (which)  *which = (ret?i:0);
#ifndef FASTPROG
	if (ret)  DEBUG(CURDLEV-2,"  -> found a del+contr element at pat=%lX (id %d) %s\n",fpatr&fpatc,i,rands);
	DEBUG(CURDLEV-1,"\t(dispminor stat: hash %d, rej %d, rej rowf %d, rej colf %d)\n",callhash,rejhash,rejrowf,rejcolf);
	if (ret && ch>=0 && IFRANDDEBUG(355)) {		/* (some paranoic tests...) */
		ee = ematrix_removematid_contr(em,i);
		j = struct_hasminorlist(ee,tghl);  dispose_ematrix(ee);
		ee = ematrix_removematid_del(em,i);
		jj = struct_hasminorlist(ee,tghl);  dispose_ematrix(ee);
		if (!j || !jj)  {PROGERROR("The del+contr element at pat=%lX (id %d) is not correct!",fpatr&fpatc,i);}
	}
#endif
	return ret? 1:0;
}

/*********
int	struct_hasdelcontr_one(int ch, ematrix *em, ematrix *tgh) {
	ematrix	*tghl[2];
	************
	tghl[0] = tgh;  tghl[1] = NULL;
	return estruct_hasdelcontr_ext(ch,em,tghl);
}********/



















/******************	Others ...................	*******************/
/**************************************************************************/


#undef CURDLEV
#define CURDLEV		5


/**
 * 
 * 
 * 
**/




































